@@ -30,6 +30,7 @@ struct lysp_submodule;
 struct sr_ds_handle;
 struct sr_mod_info_s;
 struct sr_mod_info_mod_s;
+struct sr_mod_info_xpath_s;
 struct srplg_ds_s;
 struct srplg_ntf_s;
 
@@ -1357,7 +1358,8 @@ struct lyd_node *sr_module_data_unlink(struct lyd_node **data, const struct lys_
  * @return err_info, NULL on success.
  */
 sr_error_info_t *sr_module_file_data_append(const struct lys_module *ly_mod, const struct sr_ds_handle_s *ds_handle[],
-        sr_datastore_t ds, sr_cid_t cid, uint32_t sid, const char **xpaths, uint32_t xpath_count, struct lyd_node **data);
+        sr_datastore_t ds, sr_cid_t cid, uint32_t sid, const struct sr_mod_info_xpath_s *xpaths, uint32_t xpath_count,
+        struct lyd_node **data);
 
 /**
  * @brief Learn CIDs and PIDs of all the live connections.
