@@ -108,20 +108,23 @@ static void
 error_print(int sr_error, const char *format, ...)
 {
     va_list ap;
-    char msg[2048];
+    char *msg = NULL, *nomem_msg = "error_print: Out of memory\n";
 
     if (!sr_error) {
-        sprintf(msg, "sysrepocfg error: %s\n", format);
+        if (asprintf(&msg, "sysrepocfg error: %s\n", format) == -1) {
+            msg = nomem_msg;
+        }
     } else {
-        sprintf(msg, "sysrepocfg error: %s (%s)\n", format, sr_strerror(sr_error));
+        if (asprintf(&msg, "sysrepocfg error: %s (%s)\n", format, sr_strerror(sr_error)) == -1) {
+            msg = nomem_msg;
+        }
     }
 
     va_start(ap, format);
     vfprintf(stderr, msg, ap);
     va_end(ap);
-
-    if (log_level < SR_LL_INF) {
-        fprintf(stderr, "For more details you may try to increase the verbosity up to \"-v3\".\n");
+    if (msg != nomem_msg) {
+        free(msg);
     }
 }
 
@@ -490,7 +493,7 @@ op_rpc(sr_session_ctx_t *sess, const char *file_path, const char *editor, LYD_FO
     const struct ly_ctx *ly_ctx;
     char tmp_file[23];
     int r, rc = EXIT_SUCCESS;
-    struct lyd_node *input, *node;
+    struct lyd_node *input;
     sr_data_t *output;
 
     if (!file_path && editor) {
@@ -526,13 +529,7 @@ op_rpc(sr_session_ctx_t *sess, const char *file_path, const char *editor, LYD_FO
     }
 
     /* print output if any */
-    LY_LIST_FOR(lyd_child(output->tree), node) {
-        if (!(node->flags & LYD_DEFAULT)) {
-            break;
-        }
-    }
-
-    if (node) {
+    if (output) {
         lyd_print_file(stdout, output->tree, format, wd_opt);
     }
     sr_release_data(output);
@@ -1112,6 +1109,9 @@ main(int argc, char **argv)
         error_print(0, "Internal");
         break;
     }
+    if (rc && (log_level < SR_LL_INF)) {
+        fprintf(stderr, "For more details you may try to increase the verbosity up to \"-v3\".\n");
+    }
 
 cleanup:
     sr_session_stop(sess);
