@@ -65,6 +65,7 @@
 #include "../modules/ietf_netconf_yang.h"
 #include "../modules/ietf_origin_yang.h"
 #include "../modules/sysrepo_monitoring_yang.h"
+#include "../modules/sysrepo_notifications_yang.h"
 #include "../modules/sysrepo_plugind_yang.h"
 
 /**
@@ -893,7 +894,11 @@ sr_lydmods_print(struct lyd_node **sr_mods)
     lyd_change_term_bin(node, &hash, sizeof hash);
 
     /* store the data using the internal JSON plugin */
-    if ((err_info = srpds_json.store_cb(sr_ly_mod, SR_DS_STARTUP, 0, 0, NULL, *sr_mods, NULL))) {
+    if ((err_info = srpds_json.store_prepare_cb(sr_ly_mod, SR_DS_STARTUP, 0, 0, NULL, *sr_mods, NULL))) {
+        return err_info;
+    }
+
+    if ((err_info = srpds_json.store_commit_cb(sr_ly_mod, SR_DS_STARTUP, 0, 0, NULL, *sr_mods, NULL))) {
         return err_info;
     }
 
@@ -1000,6 +1005,9 @@ sr_lydmods_create(sr_conn_ctx_t *conn, struct ly_ctx *ly_ctx, struct lyd_node **
     /* install sysrepo-plugind */
     SR_INSTALL_INT_MOD(ly_ctx, sysrepo_plugind_yang, 0, new_mods, new_mod_count);
 
+    /* install sysrepo-notifications */
+    SR_INSTALL_INT_MOD(ly_ctx, sysrepo_notifications_yang, 0, new_mods, new_mod_count);
+
     /* install ietf-netconf (implemented dependency) and ietf-netconf-with-defaults */
     SR_INSTALL_INT_MOD(ly_ctx, ietf_netconf_yang, 1, new_mods, new_mod_count);
     SR_INSTALL_INT_MOD(ly_ctx, ietf_netconf_with_defaults_yang, 0, new_mods, new_mod_count);
@@ -1350,11 +1358,14 @@ cleanup:
 
 sr_error_info_t *
 sr_lydmods_change_chng_feature(const struct ly_ctx *ly_ctx, const struct lys_module *old_mod,
-        const struct ly_ctx *new_ctx, const char *feat_name, int enable, sr_conn_ctx_t *conn, struct lyd_node **sr_mods)
+        const struct ly_ctx *new_ctx, const struct ly_set *feat_set, int enable, sr_conn_ctx_t *conn,
+        struct lyd_node **sr_mods)
 {
     sr_error_info_t *err_info = NULL;
+    const char *feat_name;
     struct lyd_node *sr_mod, *node;
     char *path = NULL;
+    uint32_t i;
 
     *sr_mods = NULL;
 
@@ -1373,27 +1384,31 @@ sr_lydmods_change_chng_feature(const struct ly_ctx *ly_ctx, const struct lys_mod
     }
     SR_CHECK_INT_GOTO(!sr_mod, err_info, cleanup);
 
-    if (enable) {
-        /* add enabled feature */
-        if ((err_info = sr_lyd_new_term(sr_mod, NULL, "enabled-feature", feat_name))) {
-            goto cleanup;
-        }
+    for (i = 0; i < feat_set->count; ++i) {
+        feat_name = feat_set->objs[i];
 
-        SR_LOG_INF("Module \"%s\" feature \"%s\" enabled.", old_mod->name, feat_name);
-    } else {
-        /* find and free the enabled feature */
-        free(path);
-        if (asprintf(&path, "enabled-feature[.='%s']", feat_name) == -1) {
-            SR_ERRINFO_MEM(&err_info);
-            goto cleanup;
-        }
-        if ((err_info = sr_lyd_find_path(sr_mod, path, 0, &node))) {
-            goto cleanup;
-        }
-        SR_CHECK_INT_GOTO(!node, err_info, cleanup);
-        lyd_free_tree(node);
+        if (enable) {
+            /* add enabled feature */
+            if ((err_info = sr_lyd_new_term(sr_mod, NULL, "enabled-feature", feat_name))) {
+                goto cleanup;
+            }
 
-        SR_LOG_INF("Module \"%s\" feature \"%s\" disabled.", old_mod->name, feat_name);
+            SR_LOG_INF("Module \"%s\" feature \"%s\" enabled.", old_mod->name, feat_name);
+        } else {
+            /* find and free the enabled feature */
+            free(path);
+            if (asprintf(&path, "enabled-feature[.='%s']", feat_name) == -1) {
+                SR_ERRINFO_MEM(&err_info);
+                goto cleanup;
+            }
+            if ((err_info = sr_lyd_find_path(sr_mod, path, 0, &node))) {
+                goto cleanup;
+            }
+            SR_CHECK_INT_GOTO(!node, err_info, cleanup);
+            lyd_free_tree(node);
+
+            SR_LOG_INF("Module \"%s\" feature \"%s\" disabled.", old_mod->name, feat_name);
+        }
     }
 
     /* delete all dependencies */
