@@ -4,8 +4,8 @@
  * @brief common routines for database plugins
  *
  * @copyright
- * Copyright (c) 2021 - 2024 Deutsche Telekom AG.
- * Copyright (c) 2021 - 2024 CESNET, z.s.p.o.
+ * Copyright (c) 2021 - 2025 Deutsche Telekom AG.
+ * Copyright (c) 2021 - 2025 CESNET, z.s.p.o.
  *
  * This source code is licensed under BSD 3-Clause License (the "License").
  * You may not use this file except in compliance with the License.
@@ -268,23 +268,13 @@ srpds_cont_set_dflt(struct lyd_node *node)
     }
 }
 
-sr_error_info_t *
-srpds_get_predicate(const char *plg_name, const struct lyd_node *node, const char **predicate, char **standard, char **no_predicate)
+const char *
+srpds_get_predicate(const char *path, const char *path_no_pred)
 {
-    sr_error_info_t *err_info = NULL;
-
-    *standard = lyd_path(node, LYD_PATH_STD, NULL, 0);
-    if (!*standard) {
-        ERRINFO(&err_info, plg_name, SR_ERR_LY, "lyd_path()", "");
-        return err_info;
+    if (path_no_pred) {
+        return path + strlen(path_no_pred);
     }
-    *no_predicate = lyd_path(node, LYD_PATH_STD_NO_LAST_PRED, NULL, 0);
-    if (!*no_predicate) {
-        ERRINFO(&err_info, plg_name, SR_ERR_LY, "lyd_path()", "");
-        return err_info;
-    }
-    *predicate = *standard + strlen(*no_predicate);
-    return err_info;
+    return NULL;
 }
 
 sr_error_info_t *
@@ -397,7 +387,7 @@ cleanup:
 }
 
 sr_error_info_t *
-srpds_escape_string(const char *plg_name, const char *string, char **escaped_string)
+srpds_escape_string(const char *plg_name, const char *string, char escape_character, char **escaped_string)
 {
     sr_error_info_t *err_info = NULL;
     uint32_t i, count, len = strlen(string);
@@ -412,7 +402,7 @@ srpds_escape_string(const char *plg_name, const char *string, char **escaped_str
                 ((string[i] >= ':') && (string[i] <= '@')) ||
                 ((string[i] >= '[') && (string[i] <= '`')) ||
                 ((string[i] >= '{') && (string[i] <= '~'))) {
-            (*escaped_string)[count] = '\\';
+            (*escaped_string)[count] = escape_character;
             ++count;
         }
         (*escaped_string)[count] = string[i];
@@ -422,6 +412,44 @@ srpds_escape_string(const char *plg_name, const char *string, char **escaped_str
     return err_info;
 }
 
+sr_error_info_t *
+srpds_find_node(const char *plg_name, const struct lyd_node *node, const struct lyd_node *tree, struct lyd_node **match)
+{
+    sr_error_info_t *err_info = NULL;
+    char *path = NULL;
+
+    if (node->parent) {
+        /* path to the parent node */
+        path = lyd_path((const struct lyd_node *)node->parent, LYD_PATH_STD, NULL, 0);
+        if (!path) {
+            ERRINFO(&err_info, plg_name, SR_ERR_LY, "lyd_path()", "");
+            goto cleanup;
+        }
+
+        /* find the parent node in mod_data */
+        if (lyd_find_path(tree, path, 0, match) != LY_SUCCESS) {
+            ERRINFO(&err_info, plg_name, SR_ERR_LY, "lyd_find_path()", "");
+            goto cleanup;
+        }
+
+        /* find the node in mod_data that matches the sibling in mod_diff */
+        if (lyd_find_sibling_val(lyd_child(*match), node->schema, NULL, 0, match) != LY_SUCCESS) {
+            ERRINFO(&err_info, plg_name, SR_ERR_LY, "lyd_find_sibling_val()", "");
+            goto cleanup;
+        }
+    } else {
+        /* find the node in mod_data that matches the sibling in mod_diff */
+        if (lyd_find_sibling_val(tree, node->schema, NULL, 0, match) != LY_SUCCESS) {
+            ERRINFO(&err_info, plg_name, SR_ERR_LY, "lyd_find_sibling_val()", "");
+            goto cleanup;
+        }
+    }
+
+cleanup:
+    free(path);
+    return err_info;
+}
+
 int
 srpds_uo_elem_comp(const void *a, const void *b)
 {
@@ -519,153 +547,89 @@ srpds_cleanup_uo_lists(srpds_db_userordered_lists_t *uo_lists)
     free(uo_lists->lists);
 }
 
+static sr_error_info_t *
+srpds_find_meta_or_attr(const char *plg_name, const struct lyd_node *tree, const char *path,
+        enum srpds_db_ly_types *type, struct lyd_node **match)
+{
+    sr_error_info_t *err_info = NULL;
+    char *newpath = NULL, *name;
+    struct ly_set *meta_match_nodes = NULL;
+    uint32_t i, len;
+
+    if (lyd_find_xpath(tree, path, &meta_match_nodes) != LY_SUCCESS) {
+        ERRINFO(&err_info, plg_name, SR_ERR_LY, "lyd_find_xpath()", "");
+        goto cleanup;
+    }
+    if (meta_match_nodes->count) {
+        *type = SRPDS_DB_LY_META;
+        *match = meta_match_nodes->dnodes[0];
+        ly_set_free(meta_match_nodes, NULL);
+        return err_info;
+    }
+
+    /* since last store a schema node could have been changed to an opaque node (try to create an attribute) */
+    newpath = strdup(path);
+    if (!newpath) {
+        ERRINFO(&err_info, plg_name, SR_ERR_NO_MEMORY, "strdup()", strerror(errno));
+        goto cleanup;
+    }
+
+    srpds_get_parent_path(newpath);
+
+    if (lyd_find_path(tree, newpath, 0, match) != LY_SUCCESS) {
+        ERRINFO(&err_info, plg_name, SR_ERR_LY, "lyd_find_path()", "");
+        goto cleanup;
+    }
+
+    name = newpath + strlen(newpath) + 1;
+    len = strlen(name);
+    for (i = 0; i < len; ++i) {
+        if (name[i] == '[') {
+            name[i] = '\0';
+            break;
+        }
+    }
+
+    if (lyd_find_sibling_opaq_next(lyd_child(*match), name, match) != LY_SUCCESS) {
+        ERRINFO(&err_info, plg_name, SR_ERR_LY, "lyd_find_sibling_opaq_next()", "");
+        goto cleanup;
+    }
+
+    *type = SRPDS_DB_LY_ATTR;
+
+cleanup:
+    free(newpath);
+    ly_set_free(meta_match_nodes, NULL);
+    return err_info;
+}
+
 sr_error_info_t *
-srpds_add_conv_mod_data(const char *plg_name, const struct ly_ctx *ly_ctx, sr_datastore_t ds, const char *path,
+srpds_add_mod_data(const char *plg_name, const struct ly_ctx *ly_ctx, sr_datastore_t ds, const char *path,
         const char *name, enum srpds_db_ly_types type, const char *module_name, const char *value, int32_t valtype,
         int *dflt_flag, const char **keys, uint32_t *lengths, int64_t order, const char *path_no_pred,
         srpds_db_userordered_lists_t *uo_lists, struct lyd_node ***parent_nodes, size_t *pnodes_size, struct lyd_node **mod_data)
-{
-    sr_error_info_t *err_info = NULL;
-    const struct lys_module *node_module;
-    struct lyd_node *new_node = NULL, *parent_node = NULL;
-    struct lyd_node **tmp_pnodes = NULL;
-    uint32_t node_idx = 0;
-
-    /* get index of the node in the parent_nodes array based on its height */
-    node_idx = srpds_get_node_depth(path) - 1;
-    parent_node = node_idx ? (*parent_nodes)[node_idx - 1] : NULL;
-
-    /* get the node module */
-    if (!module_name) {
-        node_module = NULL;
-    } else if (parent_node) {
-        /* use the parent context for ext data */
-        node_module = ly_ctx_get_module_implemented(LYD_CTX(parent_node), module_name);
-    } else {
-        node_module = ly_ctx_get_module_implemented(ly_ctx, module_name);
-    }
-
-    /* create a node based on type */
-    switch (type) {
-    case SRPDS_DB_LY_CONTAINER:    /* containers */
-        if (lyd_new_inner(parent_node, node_module, name, 0, &new_node) != LY_SUCCESS) {
-            ERRINFO(&err_info, plg_name, SR_ERR_LY, "lyd_new_inner()", ly_last_logmsg());
-            goto cleanup;
-        }
-        break;
-    case SRPDS_DB_LY_LIST:     /* lists */
-    case SRPDS_DB_LY_LIST_UO:  /* user-ordered lists */
-        if (lyd_new_list3(parent_node, node_module, name, (const char **)keys, lengths, LYD_NEW_VAL_STORE_ONLY,
-                &new_node) != LY_SUCCESS) {
-            ERRINFO(&err_info, plg_name, SR_ERR_LY, "lyd_new_list3()", ly_last_logmsg());
-            goto cleanup;
-        }
-        break;
-    case SRPDS_DB_LY_TERM:         /* leafs and leaf-lists */
-    case SRPDS_DB_LY_LEAFLIST_UO:  /* user-ordered leaf-lists */
-        if (lyd_new_term(parent_node, node_module, name, value, LYD_NEW_VAL_STORE_ONLY,
-                &new_node) != LY_SUCCESS) {
-            ERRINFO(&err_info, plg_name, SR_ERR_LY, "lyd_new_term()", ly_last_logmsg());
-            goto cleanup;
-        }
-        break;
-    case SRPDS_DB_LY_ANY:   /* anydata and anyxml */
-        if (lyd_new_any(parent_node, node_module, name, value, valtype ? LYD_ANYDATA_JSON : LYD_ANYDATA_XML,
-                LYD_NEW_VAL_STORE_ONLY, &new_node) != LY_SUCCESS) {
-            ERRINFO(&err_info, plg_name, SR_ERR_LY, "lyd_new_any()", ly_last_logmsg());
-            goto cleanup;
-        }
-        break;
-    default:
-        break;
-    }
-
-    /* store new node in the parent nodes array for children */
-    if (node_idx >= *pnodes_size) {
-        tmp_pnodes = realloc(*parent_nodes, (++*pnodes_size) * sizeof **parent_nodes);
-        if (!tmp_pnodes) {
-            ERRINFO(&err_info, plg_name, SR_ERR_NO_MEMORY, "realloc()", "");
-            goto cleanup;
-        }
-        *parent_nodes = tmp_pnodes;
-    }
-    (*parent_nodes)[node_idx] = new_node;
-    if (!node_idx) {
-        if (lyd_insert_sibling(*mod_data, new_node, mod_data) != LY_SUCCESS) {
-            ERRINFO(&err_info, plg_name, SR_ERR_LY, "lyd_insert_sibling()", ly_last_logmsg());
-            goto cleanup;
-        }
-    }
-
-    /* store nodes and their orders of userordered lists and leaflists for final ordering */
-    if ((type == SRPDS_DB_LY_LIST_UO) || (type == SRPDS_DB_LY_LEAFLIST_UO)) {
-        if ((err_info = srpds_add_uo_lists(plg_name, new_node, order, path_no_pred, uo_lists))) {
-            goto cleanup;
-        }
-    }
-
-    /* for default nodes add a flag */
-    if (*dflt_flag) {
-        new_node->flags = new_node->flags | LYD_DEFAULT;
-        *dflt_flag = 0;
-        srpds_cont_set_dflt(lyd_parent(new_node));
-    }
-
-    /* for 'when' nodes add a flag */
-    switch (ds) {
-    case SR_DS_STARTUP:
-    case SR_DS_RUNNING:
-    case SR_DS_FACTORY_DEFAULT:
-        while ((*parent_nodes)[0] != new_node) {
-            if (lysc_has_when(new_node->schema)) {
-                new_node->flags |= LYD_WHEN_TRUE;
-            }
-            new_node->flags &= ~LYD_NEW;
-            new_node = lyd_parent(new_node);
-        }
-        if (lysc_has_when((*parent_nodes)[0]->schema)) {
-            (*parent_nodes)[0]->flags |= LYD_WHEN_TRUE;
-        }
-        (*parent_nodes)[0]->flags &= ~LYD_NEW;
-        break;
-    default:
-        break;
-    }
-
-cleanup:
-    return err_info;
-}
-
-sr_error_info_t *
-srpds_add_oper_mod_data(const char *plg_name, const struct ly_ctx *ly_ctx, const char *path, const char *name,
-        enum srpds_db_ly_types type, const char *module_name, const char *value, int32_t valtype, int *dflt_flag,
-        const char **keys, uint32_t *lengths, struct lyd_node ***parent_nodes, size_t *pnodes_size, struct lyd_node **mod_data)
 {
     sr_error_info_t *err_info = NULL;
     const struct lys_module *node_module;
     struct lyd_node **tmp_pnodes = NULL;
     struct lyd_node *meta_match = NULL, *new_node = NULL, *parent_node = NULL;
-    struct ly_set *meta_match_nodes = NULL;
     uint32_t node_idx = 0;
+    enum srpds_db_ly_types dbtype;
+    LY_ERR lerr = LY_SUCCESS;
 
     /* get index of the node in the parent_nodes array based on its height */
     if ((type != SRPDS_DB_LY_META) && (type != SRPDS_DB_LY_ATTR)) {
         node_idx = srpds_get_node_depth(path) - 1;
         parent_node = node_idx ? (*parent_nodes)[node_idx - 1] : NULL;
     } else { /* get node to which we want to add the metadata or attribute */
-        if (lyd_find_xpath(*mod_data, path, &meta_match_nodes) != LY_SUCCESS) {
-            ERRINFO(&err_info, plg_name, SR_ERR_LY, "lyd_find_xpath()", "");
+        if ((err_info = srpds_find_meta_or_attr(plg_name, *mod_data, path, &dbtype, &meta_match))) {
             goto cleanup;
         }
-        if (!meta_match_nodes->count) {
-            ERRINFO(&err_info, plg_name, SR_ERR_NOT_FOUND, "lyd_find_xpath()", "Path not found");
-            goto cleanup;
-        }
-        meta_match = meta_match_nodes->dnodes[0];
+        type = dbtype;
     }
 
     /* get the node module */
-    if (!module_name) {
+    if (!module_name || !strcmp(module_name, "")) {
         node_module = NULL;
     } else if (parent_node) {
         /* use the parent context for ext data */
@@ -691,34 +655,41 @@ srpds_add_oper_mod_data(const char *plg_name, const struct ly_ctx *ly_ctx, const
         }
         break;
     case SRPDS_DB_LY_CONTAINER:    /* containers */
-        if (lyd_new_inner(parent_node, node_module, name, 0, &new_node) != LY_SUCCESS) {
-            ERRINFO(&err_info, plg_name, SR_ERR_LY, "lyd_new_inner()", "");
+        lerr = lyd_new_inner(parent_node, node_module, name, 0, &new_node);
+        if ((lerr != LY_SUCCESS) && (lerr != LY_ENOTFOUND)) {
+            ERRINFO(&err_info, plg_name, SR_ERR_LY, "lyd_new_inner()", ly_last_logmsg());
             goto cleanup;
         }
         break;
-    case SRPDS_DB_LY_LIST:         /* lists */
-        if (lyd_new_list3(parent_node, node_module, name, keys, lengths, LYD_NEW_VAL_STORE_ONLY,
-                &new_node) != LY_SUCCESS) {
-            ERRINFO(&err_info, plg_name, SR_ERR_LY, "lyd_new_list3()", "");
+    case SRPDS_DB_LY_LIST:     /* lists */
+    case SRPDS_DB_LY_LIST_UO:  /* user-ordered lists */
+        lerr = lyd_new_list3(parent_node, node_module, name, (const char **)keys, lengths, LYD_NEW_VAL_STORE_ONLY,
+                &new_node);
+        if ((lerr != LY_SUCCESS) && (lerr != LY_ENOTFOUND)) {
+            ERRINFO(&err_info, plg_name, SR_ERR_LY, "lyd_new_list3()", ly_last_logmsg());
             goto cleanup;
         }
         break;
     case SRPDS_DB_LY_TERM:         /* leafs and leaf-lists */
-        if (lyd_new_term(parent_node, node_module, name, value, LYD_NEW_VAL_STORE_ONLY,
-                &new_node) != LY_SUCCESS) {
-            ERRINFO(&err_info, plg_name, SR_ERR_LY, "lyd_new_term()", "");
+    case SRPDS_DB_LY_LEAFLIST_UO:  /* user-ordered leaf-lists */
+        lerr = lyd_new_term(parent_node, node_module, name, value, LYD_NEW_VAL_STORE_ONLY,
+                &new_node);
+        if ((lerr != LY_SUCCESS) && (lerr != LY_ENOTFOUND)) {
+            ERRINFO(&err_info, plg_name, SR_ERR_LY, "lyd_new_term()", ly_last_logmsg());
             goto cleanup;
         }
         break;
-    case SRPDS_DB_LY_ANY:          /* anydata and anyxml */
-        if (lyd_new_any(parent_node, node_module, name, value, valtype ? LYD_ANYDATA_JSON : LYD_ANYDATA_XML,
-                LYD_NEW_VAL_STORE_ONLY, &new_node) != LY_SUCCESS) {
-            ERRINFO(&err_info, plg_name, SR_ERR_LY, "lyd_new_any()", "");
+    case SRPDS_DB_LY_ANY:   /* anydata and anyxml */
+        lerr = lyd_new_any(parent_node, node_module, name, value, valtype ? LYD_ANYDATA_JSON : LYD_ANYDATA_XML,
+                LYD_NEW_VAL_STORE_ONLY, &new_node);
+        if ((lerr != LY_SUCCESS) && (lerr != LY_ENOTFOUND)) {
+            ERRINFO(&err_info, plg_name, SR_ERR_LY, "lyd_new_any()", ly_last_logmsg());
             goto cleanup;
         }
         break;
     case SRPDS_DB_LY_OPAQUE:       /* opaque nodes */
-        if (lyd_new_opaq(parent_node, ly_ctx, name, value, NULL, module_name, &new_node) != LY_SUCCESS) {
+        if (lyd_new_opaq(parent_node, ly_ctx, name, value, NULL,
+                (module_name && strcmp(module_name, "")) ? module_name : lyd_node_module(parent_node)->name, &new_node) != LY_SUCCESS) {
             ERRINFO(&err_info, plg_name, SR_ERR_LY, "lyd_new_opaq()", "");
             goto cleanup;
         }
@@ -727,10 +698,19 @@ srpds_add_oper_mod_data(const char *plg_name, const struct ly_ctx *ly_ctx, const
         break;
     }
 
+    /* since last store a schema node could have been changed to an opaque node (try to create opaque) */
+    if (lerr == LY_ENOTFOUND) {
+        if (lyd_new_opaq(parent_node, ly_ctx, name, value, NULL,
+                (module_name && strcmp(module_name, "")) ? module_name : lyd_node_module(parent_node)->name, &new_node) != LY_SUCCESS) {
+            ERRINFO(&err_info, plg_name, SR_ERR_LY, "lyd_new_opaq()", "");
+            goto cleanup;
+        }
+    }
+
     if ((type != SRPDS_DB_LY_META) && (type != SRPDS_DB_LY_ATTR)) {
         /* store new node in the parent nodes array for children */
         if (node_idx >= *pnodes_size) {
-            tmp_pnodes = realloc((*parent_nodes), (++*pnodes_size) * sizeof **parent_nodes);
+            tmp_pnodes = realloc(*parent_nodes, (++*pnodes_size) * sizeof **parent_nodes);
             if (!tmp_pnodes) {
                 ERRINFO(&err_info, plg_name, SR_ERR_NO_MEMORY, "realloc()", "");
                 goto cleanup;
@@ -740,38 +720,70 @@ srpds_add_oper_mod_data(const char *plg_name, const struct ly_ctx *ly_ctx, const
         (*parent_nodes)[node_idx] = new_node;
         if (!node_idx) {
             if (lyd_insert_sibling(*mod_data, new_node, mod_data) != LY_SUCCESS) {
-                ERRINFO(&err_info, plg_name, SR_ERR_LY, "lyd_insert_sibling()", "");
+                ERRINFO(&err_info, plg_name, SR_ERR_LY, "lyd_insert_sibling()", ly_last_logmsg());
                 goto cleanup;
             }
         }
 
+        /* store nodes and their orders of userordered lists and leaflists for final ordering */
+        switch (type) {
+        case SRPDS_DB_LY_LIST_UO:
+        case SRPDS_DB_LY_LEAFLIST_UO:
+            if ((err_info = srpds_add_uo_lists(plg_name, new_node, order, path_no_pred, uo_lists))) {
+                goto cleanup;
+            }
+            break;
+        default:
+            break;
+        }
+
         /* for default nodes add a flag */
         if (*dflt_flag) {
             new_node->flags = new_node->flags | LYD_DEFAULT;
             *dflt_flag = 0;
             srpds_cont_set_dflt(lyd_parent(new_node));
         }
+
+        /* for 'when' nodes add a flag */
+        switch (ds) {
+        case SR_DS_STARTUP:
+        case SR_DS_RUNNING:
+        case SR_DS_FACTORY_DEFAULT:
+            while ((*parent_nodes)[0] != new_node) {
+                if (lysc_has_when(new_node->schema)) {
+                    new_node->flags |= LYD_WHEN_TRUE;
+                }
+                new_node->flags &= ~LYD_NEW;
+                new_node = lyd_parent(new_node);
+            }
+            if (lysc_has_when((*parent_nodes)[0]->schema)) {
+                (*parent_nodes)[0]->flags |= LYD_WHEN_TRUE;
+            }
+            (*parent_nodes)[0]->flags &= ~LYD_NEW;
+            break;
+        default:
+            break;
+        }
     }
 
 cleanup:
-    ly_set_free(meta_match_nodes, NULL);
     return err_info;
 }
 
 sr_error_info_t *
-srpds_get_values(const char *plg_name, struct lyd_node *node, const char **value, const char **prev, const char **orig_prev,
+srpds_get_values(const char *plg_name, const struct lyd_node *node, const char **value, const char **prev, const char **orig_prev,
         char **prev_pred, char **orig_prev_pred, char **any_value, int32_t *valtype)
 {
     sr_error_info_t *err_info = NULL;
 
-    /* LYD_ANYDATA_XML */
-    *valtype = 0;
-
     if (node->schema->nodetype & LYD_NODE_ANY) {
         /* these are JSON data, set valtype */
         if (((struct lyd_node_any *)node)->value_type == LYD_ANYDATA_JSON) {
             /* LYD_ANYDATA_JSON */
             *valtype = 1;
+        } else {
+            /* LYD_ANYDATA_XML */
+            *valtype = 0;
         }
 
         /* lyd_node_any */
@@ -780,7 +792,11 @@ srpds_get_values(const char *plg_name, struct lyd_node *node, const char **value
             goto cleanup;
         }
         *value = *any_value;
+    } else if (!(node->schema->flags & LYS_CONFIG_W)) {
+        /* state data */
+        *value = lyd_get_value(node);
     } else if (lysc_is_userordered(node->schema)) {
+        /* configuration userordered lists and leaf-lists */
         /* get value of the previous node */
         if (node->schema->nodetype == LYS_LEAFLIST) {
             *prev = lyd_get_meta_value(lyd_find_meta(node->meta, NULL, "yang:value"));
@@ -805,6 +821,7 @@ srpds_get_values(const char *plg_name, struct lyd_node *node, const char **value
             *orig_prev_pred = (char *)*orig_prev;
         }
     } else {
+        /* the rest */
         *value = lyd_get_value(node);
     }
 
@@ -813,7 +830,7 @@ cleanup:
 }
 
 void
-srpds_cleanup_values(struct lyd_node *node, const char *prev, const char *orig_prev, char **prev_pred, char **orig_prev_pred,
+srpds_cleanup_values(const struct lyd_node *node, const char *prev, const char *orig_prev, char **prev_pred, char **orig_prev_pred,
         char **any_value)
 {
     free(*any_value);
