@@ -4,8 +4,8 @@
  * @brief common routines for database plugins header
  *
  * @copyright
- * Copyright (c) 2021 - 2024 Deutsche Telekom AG.
- * Copyright (c) 2021 - 2024 CESNET, z.s.p.o.
+ * Copyright (c) 2021 - 2025 Deutsche Telekom AG.
+ * Copyright (c) 2021 - 2025 CESNET, z.s.p.o.
  *
  * This source code is licensed under BSD 3-Clause License (the "License").
  * You may not use this file except in compliance with the License.
@@ -133,17 +133,13 @@ sr_error_info_t *srpds_get_modif_path(const char *plg_name, const char *path, ch
 void srpds_cont_set_dflt(struct lyd_node *node);
 
 /**
- * @brief Get path, predicate and path without the predicate of a data node.
+ * @brief Get predicate of a data node.
  *
- * @param[in] plg_name Plugin name.
- * @param[in] node Given data node.
- * @param[out] predicate Predicate of the data node.
- * @param[out] standard Path of the data node. Should be freed.
- * @param[out] no_predicate Path without the predicate of the data node. Should be freed.
- * @return NULL on success;
- * @return Sysrepo error info on error.
+ * @param[in] path Path of the data node.
+ * @param[in] path_no_pred Path without the predicate of the data node.
+ * @return Predicate.
  */
-sr_error_info_t *srpds_get_predicate(const char *plg_name, const struct lyd_node *node, const char **predicate, char **standard, char **no_predicate);
+const char *srpds_get_predicate(const char *path, const char *path_no_pred);
 
 /**
  * @brief Get the name of the current process user.
@@ -172,11 +168,24 @@ sr_error_info_t *srpds_gid2grp(const char *plg_name, gid_t gid, char **group);
  *
  * @param[in] plg_name Plugin name.
  * @param[in] string String to escape.
+ * @param[in] escape_character Character to escape with ('\' or '%'). '%' is used to escape in Lua.
  * @param[out] escaped_string Escaped string.
  * @return NULL on success;
  * @return Sysrepo error info on error.
  */
-sr_error_info_t *srpds_escape_string(const char *plg_name, const char *string, char **escaped_string);
+sr_error_info_t *srpds_escape_string(const char *plg_name, const char *string, char escape_character, char **escaped_string);
+
+/**
+ * @brief Find the node @p node from one tree in another tree @p tree and store it in @p match .
+ *
+ * @param[in] plg_name Plugin name.
+ * @param[in] node Node to find.
+ * @param[in] tree Data tree to search.
+ * @param[out] match Found node. NULL in case it was not found.
+ * @return NULL on success;
+ * @return Sysrepo error info on error.
+ */
+sr_error_info_t *srpds_find_node(const char *plg_name, const struct lyd_node *node, const struct lyd_node *tree, struct lyd_node **match);
 
 /**
  * @brief Compare function for qsort comparing elements of userordered lists and leaflists.
@@ -219,7 +228,7 @@ sr_error_info_t *srpds_order_uo_lists(const char *plg_name, const srpds_db_usero
 void srpds_cleanup_uo_lists(srpds_db_userordered_lists_t *uo_lists);
 
 /**
- * @brief Add a new node to the data tree in conventional datastore.
+ * @brief Add a new node to the data tree in the datastore.
  *
  * @param[in] plg_name Plugin name.
  * @param[in] ly_ctx Context to use.
@@ -242,37 +251,12 @@ void srpds_cleanup_uo_lists(srpds_db_userordered_lists_t *uo_lists);
  * @return NULL on success;
  * @return Sysrepo error info on error.
  */
-sr_error_info_t *srpds_add_conv_mod_data(const char *plg_name, const struct ly_ctx *ly_ctx, sr_datastore_t ds,
+sr_error_info_t *srpds_add_mod_data(const char *plg_name, const struct ly_ctx *ly_ctx, sr_datastore_t ds,
         const char *path, const char *name, enum srpds_db_ly_types type, const char *module_name, const char *value,
         int32_t valtype, int *dflt_flag, const char **keys, uint32_t *lengths, int64_t order, const char *path_no_pred,
         srpds_db_userordered_lists_t *uo_lists, struct lyd_node ***parent_nodes, size_t *pnodes_size,
         struct lyd_node **mod_data);
 
-/**
- * @brief Add a new node to the data tree in operational datastore.
- *
- * @param[in] plg_name Plugin name.
- * @param[in] ly_ctx Context to use.
- * @param[in] path Path to node.
- * @param[in] name Name of the node.
- * @param[in] type Type of the node.
- * @param[in] module_name Module name of the node.
- * @param[in] value Value of the node.
- * @param[in] valtype Type of the value (XML=0 or JSON=1).
- * @param[in,out] dflt_flag Whether the node has default value.
- * @param[in] keys Array of the keys of the node (list instance).
- * @param[in] lengths Array of the lengths of the @p keys .
- * @param[in,out] parent_nodes Potential parent nodes of the new node.
- * @param[in,out] pnodes_size Size of the @p parent_nodes .
- * @param[in,out] mod_data Data tree to insert the new node into.
- * @return NULL on success;
- * @return Sysrepo error info on error.
- */
-sr_error_info_t *srpds_add_oper_mod_data(const char *plg_name, const struct ly_ctx *ly_ctx, const char *path,
-        const char *name, enum srpds_db_ly_types type, const char *module_name, const char *value,
-        int32_t valtype, int *dflt_flag, const char **keys, uint32_t *lengths, struct lyd_node ***parent_nodes,
-        size_t *pnodes_size, struct lyd_node **mod_data);
-
 /**
  * @brief Get all the values associated with the node.
  *
@@ -288,7 +272,7 @@ sr_error_info_t *srpds_add_oper_mod_data(const char *plg_name, const struct ly_c
  * @return NULL on success;
  * @return Sysrepo error info on error.
  */
-sr_error_info_t *srpds_get_values(const char *plg_name, struct lyd_node *node, const char **value, const char **prev,
+sr_error_info_t *srpds_get_values(const char *plg_name, const struct lyd_node *node, const char **value, const char **prev,
         const char **orig_prev, char **prev_pred, char **orig_prev_pred, char **any_value, int32_t *valtype);
 
 /**
@@ -301,7 +285,7 @@ sr_error_info_t *srpds_get_values(const char *plg_name, struct lyd_node *node, c
  * @param[in,out] orig_prev_pred Original value of the node before this node in predicate.
  * @param[in,out] any_value Value of the type 'any value'.
  */
-void srpds_cleanup_values(struct lyd_node *node, const char *prev, const char *orig_prev, char **prev_pred,
+void srpds_cleanup_values(const struct lyd_node *node, const char *prev, const char *orig_prev, char **prev_pred,
         char **orig_prev_pred, char **any_value);
 
 #endif /* _COMMON_DB_H */
