@@ -114,6 +114,9 @@ srpntf_read_notif(int notif_fd, struct ly_ctx *ly_ctx, struct lyd_node **notif)
         goto cleanup;
     }
 
+    /* size sanity check */
+    assert(notif_json_len <= (UINT32_MAX - 1));
+
     /* read the notification */
     notif_json = malloc(notif_json_len + 1);
     if (!notif_json) {
@@ -212,8 +215,8 @@ cleanup:
  * @param[in] mod_name Module name.
  * @param[in] from_ts Earliest stored notification.
  * @param[in] to_ts Latest stored notification.
- * @param[out] file_from_ts Found file earliest notification.
- * @param[out] file_to_ts Found file latest notification.
+ * @param[out] file_from_ts Found file earliest notification, 0 if no matching file found.
+ * @param[out] file_to_ts Found file latest notification, 0 if no matching file found.
  * @return err_info, NULL on success.
  */
 static sr_error_info_t *
@@ -456,6 +459,11 @@ srpntf_json_store(const struct lys_module *mod, const struct lyd_node *notif, co
 
             /* we are done */
             goto cleanup;
+        } else if ((notif_ts->tv_sec == from_ts) && (notif_ts->tv_sec == to_ts)) {
+            srplg_log_errinfo(&err_info, srpntf_name, NULL, SR_ERR_OPERATION_FAILED,
+                    "Notification file %" PRId64 "-%" PRId64 " exceeded the maximum size of %d KB.",
+                    (int64_t)from_ts, (int64_t)to_ts, SRPJSON_NOTIF_FILE_MAX_SIZE);
+            goto cleanup;
         }
 
         /* we will create a new file, close this one */
@@ -793,6 +801,36 @@ cleanup:
     return err_info;
 }
 
+static void
+srpntf_json_destroy(const struct lys_module *mod)
+{
+    sr_error_info_t *err_info = NULL;
+    time_t file_from, file_to;
+    char *path;
+
+    /* find the first notification file */
+    if ((err_info = srpntf_find_file(mod->name, 1, 1, &file_from, &file_to))) {
+        goto cleanup;
+    }
+
+    while (file_from && file_to) {
+        /* unlink it */
+        if ((err_info = srpjson_get_notif_path(srpntf_name, mod->name, file_from, file_to, &path))) {
+            goto cleanup;
+        }
+        unlink(path);
+        free(path);
+
+        /* find next notification file */
+        if ((err_info = srpntf_find_file(mod->name, file_from, file_to, &file_from, &file_to))) {
+            goto cleanup;
+        }
+    }
+
+cleanup:
+    srplg_errinfo_free(&err_info);
+}
+
 const struct srplg_ntf_s srpntf_json = {
     .name = srpntf_name,
     .enable_cb = srpntf_json_enable,
@@ -803,4 +841,5 @@ const struct srplg_ntf_s srpntf_json = {
     .access_set_cb = srpntf_json_access_set,
     .access_get_cb = srpntf_json_access_get,
     .access_check_cb = srpntf_json_access_check,
+    .destroy_cb = srpntf_json_destroy,
 };
