@@ -870,7 +870,6 @@ sr_shmmod_del_module_sess_oper_data(sr_conn_ctx_t *conn, const struct lys_module
 {
     sr_error_info_t *err_info = NULL;
     const struct sr_ds_handle_s *oper_ds_handle;
-    struct lyd_node *mod_diff = NULL, *node;
 
     if (has_data) {
         /* get DS handle */
@@ -879,29 +878,13 @@ sr_shmmod_del_module_sess_oper_data(sr_conn_ctx_t *conn, const struct lys_module
             goto cleanup;
         }
 
-        /* create a diff only if the plugin requires it to store new data */
-        if (oper_ds_handle->plugin->oper_store_require_diff) {
-            /* load oper data */
-            if ((err_info = oper_ds_handle->plugin->load_cb(ly_mod, SR_DS_OPERATIONAL, cid, sid, NULL, 0, oper_ds_handle->plg_data,
-                    &mod_diff))) {
-                goto cleanup;
-            }
-
-            /* make into diff */
-            LY_LIST_FOR(mod_diff, node) {
-                if ((err_info = sr_diff_set_oper(node, "delete"))) {
-                    goto cleanup;
-                }
-            }
-        }
-
         /* store the empty data */
-        if ((err_info = oper_ds_handle->plugin->store_prepare_cb(ly_mod, SR_DS_OPERATIONAL, cid, sid, mod_diff,
+        if ((err_info = oper_ds_handle->plugin->store_prepare_cb(ly_mod, SR_DS_OPERATIONAL, cid, sid, NULL,
                 NULL, oper_ds_handle->plg_data))) {
             goto cleanup;
         }
 
-        if ((err_info = oper_ds_handle->plugin->store_commit_cb(ly_mod, SR_DS_OPERATIONAL, cid, sid, mod_diff,
+        if ((err_info = oper_ds_handle->plugin->store_commit_cb(ly_mod, SR_DS_OPERATIONAL, cid, sid, NULL,
                 NULL, oper_ds_handle->plg_data))) {
             goto cleanup;
         }
@@ -913,7 +896,6 @@ sr_shmmod_del_module_sess_oper_data(sr_conn_ctx_t *conn, const struct lys_module
     }
 
 cleanup:
-    lyd_free_siblings(mod_diff);
     return err_info;
 }
 
@@ -1161,7 +1143,7 @@ sr_shmmod_collect_deps_lref(const char *target_path, const char *target_module,
     SR_CHECK_INT_RET(!ly_mod, err_info);
 
     /* add dependency */
-    if ((err_info = sr_modinfo_add(ly_mod, target_path, 0, 0, mod_info))) {
+    if ((err_info = sr_modinfo_add(ly_mod, target_path, 0, 0, 0, mod_info))) {
         return err_info;
     }
 
@@ -1206,7 +1188,7 @@ sr_shmmod_collect_deps_instid(const char *source_path, const char *default_targe
             SR_CHECK_INT_GOTO(!ly_mod, err_info, cleanup);
 
             /* add module */
-            if ((err_info = sr_modinfo_add(ly_mod, val_str, 0, 0, mod_info))) {
+            if ((err_info = sr_modinfo_add(ly_mod, val_str, 0, 0, 0, mod_info))) {
                 goto cleanup;
             }
         }
@@ -1217,7 +1199,7 @@ sr_shmmod_collect_deps_instid(const char *source_path, const char *default_targe
         free(str);
         SR_CHECK_INT_GOTO(!ly_mod, err_info, cleanup);
 
-        if ((err_info = sr_modinfo_add(ly_mod, default_target_path, 0, 0, mod_info))) {
+        if ((err_info = sr_modinfo_add(ly_mod, default_target_path, 0, 0, 0, mod_info))) {
             goto cleanup;
         }
     }
@@ -1252,7 +1234,7 @@ sr_shmmod_collect_deps_xpath(const char *expr, char *mod_shm_addr, off_t *target
         SR_CHECK_INT_RET(!ly_mod, err_info);
 
         /* add dependency */
-        if ((err_info = sr_modinfo_add(ly_mod, expr, 0, 0, mod_info))) {
+        if ((err_info = sr_modinfo_add(ly_mod, expr, 0, 0, 0, mod_info))) {
             return err_info;
         }
     }
@@ -1311,7 +1293,7 @@ sr_shmmod_lock(struct sr_mod_lock_s *shm_lock, uint32_t timeout_ms, sr_lock_mode
 {
     sr_error_info_t *err_info = NULL, *tmp_err;
     int ds_locked;
-    uint32_t sleep_ms;
+    uint32_t sleep_ms, ds_lock_sid = 0;
     sr_cid_t dead_cid;
 
     if (!timeout_ms) {
@@ -1354,6 +1336,8 @@ ds_lock_retry:
             ds_locked = 0;
         }
 
+        ds_lock_sid = shm_lock->ds_lock_sid;
+
         /* DS UNLOCK */
         sr_munlock(&shm_lock->ds_lock);
     }
@@ -1397,7 +1381,7 @@ revert_lock:
     } else {
         /* timeout elapsed */
         sr_errinfo_new(&err_info, SR_ERR_LOCKED, "Module \"%s\" is DS-locked by session %" PRIu32 ".",
-                mod_name, shm_lock->ds_lock_sid);
+                mod_name, ds_lock_sid);
     }
 
 cleanup:
