@@ -2255,6 +2255,9 @@ cleanup:
 sr_error_info_t *
 sr_nacm_check_yp_change_begin(const char *nacm_user, char ***groups, uint32_t *group_count)
 {
+    *groups = NULL;
+    *group_count = 0;
+
     /* NACM LOCK */
     pthread_mutex_lock(&nacm.lock);
 
@@ -2263,6 +2266,11 @@ sr_nacm_check_yp_change_begin(const char *nacm_user, char ***groups, uint32_t *g
         return NULL;
     }
 
+    /* NACM is off */
+    if (!nacm.enabled) {
+        return NULL;
+    }
+
     return sr_nacm_collect_groups(nacm_user, groups, group_count);
 }
 
@@ -2279,6 +2287,16 @@ sr_nacm_check_yp_change_target(const char *nacm_user, char **groups, uint32_t gr
         goto cleanup;
     }
 
+    /* 1) NACM is off */
+    if (!nacm.enabled) {
+        goto cleanup;
+    }
+
+    /* 2) recovery session allowed */
+    if (!strcmp(nacm_user, SR_NACM_RECOVERY_USER)) {
+        goto cleanup;
+    }
+
     /* check the node */
     if ((err_info = sr_nacm_allowed_node(NULL, path, schema, SR_NACM_OP_READ, groups, group_count, nacm_user, &access,
             NULL, NULL))) {
