@@ -16,6 +16,7 @@
 
 #define _GNU_SOURCE
 
+#include "compat.h"
 #include "values.h"
 
 #include <inttypes.h>
@@ -24,12 +25,14 @@
 #include <unistd.h>
 
 #include "common.h"
-#include "compat.h"
 #include "config.h"
 #include "log.h"
+#include "ly_wrap.h"
 
 /** get the larger item */
-#define MAX(a, b) ((a) > (b) ? (a) : (b))
+#ifndef MAX
+# define MAX(a, b) ((a) > (b) ? (a) : (b))
+#endif
 
 /**
  * @brief Type of the destination for the print operation.
@@ -45,9 +48,11 @@ typedef enum sr_print_type_e {
  */
 typedef struct sr_print_ctx_s {
     sr_print_type_t type;
+
     union {
         int fd;
         FILE *stream;
+
         struct {
             char *buf;
             size_t len;
@@ -60,7 +65,7 @@ typedef struct sr_print_ctx_s {
  * @brief Returns pointer to the string where data of given string type is stored.
  */
 static char **
-sr_val_str_data_ptr(sr_val_t *value, sr_type_t type)
+sr_val_str_data_ptr(sr_val_t *value, sr_val_type_t type)
 {
     switch (type) {
     case SR_BINARY_T:
@@ -224,7 +229,7 @@ sr_val_build_xpath(sr_val_t *value, const char *format, ...)
 }
 
 API int
-sr_val_set_str_data(sr_val_t *value, sr_type_t type, const char *string_val)
+sr_val_set_str_data(sr_val_t *value, sr_val_type_t type, const char *string_val)
 {
     char **str_to_edit = NULL;
 
@@ -238,7 +243,7 @@ sr_val_set_str_data(sr_val_t *value, sr_type_t type, const char *string_val)
 }
 
 static int
-sr_val_build_str_data_va(sr_val_t *value, sr_type_t type, const char *format, va_list args)
+sr_val_build_str_data_va(sr_val_t *value, sr_val_type_t type, const char *format, va_list args)
 {
     char **str_to_edit = NULL;
     int rc = SR_ERR_OK;
@@ -255,7 +260,7 @@ sr_val_build_str_data_va(sr_val_t *value, sr_type_t type, const char *format, va
 }
 
 API int
-sr_val_build_str_data(sr_val_t *value, sr_type_t type, const char *format, ...)
+sr_val_build_str_data(sr_val_t *value, sr_val_type_t type, const char *format, ...)
 {
     va_list arg_list;
     int rc = SR_ERR_OK;
@@ -372,6 +377,80 @@ cleanup:
     return rc;
 }
 
+static int
+sr_cmp_str(const char *c1, const char *c2)
+{
+    if ((c1 != NULL) && (c2 == NULL)) {
+        return -1;
+    } else if ((c1 == NULL) && (c2 != NULL)) {
+        return 1;
+    } else if ((c1 == NULL) && (c2 == NULL)) {
+        return 0;
+    }
+
+    return strcmp(c1, c2);
+}
+
+API int
+sr_equal_val(const sr_val_t *value1, const sr_val_t *value2)
+{
+    if ((value1->type != value2->type) ||
+            (0 != sr_cmp_str(value1->origin, value2->origin)) ||
+            (0 != sr_cmp_str(value1->xpath, value2->xpath)) ||
+            (value1->dflt != value2->dflt)) {
+        return 0;
+    }
+
+    switch (value1->type) {
+    case SR_BINARY_T:
+        return 0 == sr_cmp_str(value1->data.binary_val, value2->data.binary_val);
+    case SR_BITS_T:
+        return 0 == sr_cmp_str(value1->data.bits_val, value2->data.bits_val);
+    case SR_ENUM_T:
+        return 0 == sr_cmp_str(value1->data.enum_val, value2->data.enum_val);
+    case SR_IDENTITYREF_T:
+        return 0 == sr_cmp_str(value1->data.identityref_val, value2->data.identityref_val);
+    case SR_INSTANCEID_T:
+        return 0 == sr_cmp_str(value1->data.instanceid_val, value2->data.instanceid_val);
+    case SR_STRING_T:
+        return 0 == sr_cmp_str(value1->data.string_val, value2->data.string_val);
+    case SR_ANYXML_T:
+        return 0 == sr_cmp_str(value1->data.anyxml_val, value2->data.anyxml_val);
+    case SR_ANYDATA_T:
+        return 0 == sr_cmp_str(value1->data.anydata_val, value2->data.anydata_val);
+    case SR_BOOL_T:
+        return value1->data.bool_val == value2->data.bool_val;
+    case SR_DECIMAL64_T:
+        return value1->data.decimal64_val == value2->data.decimal64_val;
+    case SR_INT8_T:
+        return value1->data.int8_val == value2->data.int8_val;
+    case SR_INT16_T:
+        return value1->data.int16_val == value2->data.int16_val;
+    case SR_INT32_T:
+        return value1->data.int32_val == value2->data.int32_val;
+    case SR_INT64_T:
+        return value1->data.int64_val == value2->data.int64_val;
+    case SR_UINT8_T:
+        return value1->data.uint8_val == value2->data.uint8_val;
+    case SR_UINT16_T:
+        return value1->data.uint16_val == value2->data.uint16_val;
+    case SR_UINT32_T:
+        return value1->data.uint32_val == value2->data.uint32_val;
+    case SR_UINT64_T:
+        return value1->data.uint64_val == value2->data.uint64_val;
+    case SR_UNKNOWN_T:
+    case SR_CONTAINER_T:
+    case SR_CONTAINER_PRESENCE_T:
+    case SR_LIST_T:
+    case SR_LEAF_EMPTY_T:
+    case SR_NOTIFICATION_T:
+        /* No value to compare */
+        break;
+    }
+
+    return 1;
+}
+
 static int
 sr_print(sr_print_ctx_t *print_ctx, const char *format, ...)
 {
@@ -848,8 +927,7 @@ sr_tree_to_val(const struct lyd_node *data, const char *path, sr_val_t **value)
 
     *value = NULL;
 
-    if (lyd_find_xpath(data, path, &set)) {
-        sr_errinfo_new_ly(&err_info, LYD_CTX(data));
+    if ((err_info = sr_lyd_find_xpath(data, path, &set))) {
         goto cleanup;
     } else if (!set->count) {
         /* Not building err_info to avoid error logs when no item found */
@@ -864,11 +942,10 @@ sr_tree_to_val(const struct lyd_node *data, const char *path, sr_val_t **value)
     *value = malloc(sizeof **value);
     SR_CHECK_MEM_GOTO(!*value, err_info, cleanup);
 
-    if ((err_info = sr_val_ly2sr(set->dnodes[0], *value))) {
+    if ((err_info = sr_val_ly2sr(set->dnodes[0], 1, *value))) {
         goto cleanup;
     }
 
-    /* success */
 cleanup:
     ly_set_free(set, NULL);
     return sr_api_ret(NULL, err_info);
@@ -886,8 +963,7 @@ sr_tree_to_values(const struct lyd_node *data, const char *xpath, sr_val_t **val
     *values = NULL;
     *value_cnt = 0;
 
-    if (lyd_find_xpath(data, xpath, &set)) {
-        sr_errinfo_new_ly(&err_info, LYD_CTX(data));
+    if ((err_info = sr_lyd_find_xpath(data, xpath, &set))) {
         goto cleanup;
     } else if (!set->count) {
         /* Not building err_info to avoid error logs when no item found */
@@ -902,14 +978,13 @@ sr_tree_to_values(const struct lyd_node *data, const char *xpath, sr_val_t **val
                 continue;
             }
 
-            if ((err_info = sr_val_ly2sr(set->dnodes[i], *values + *value_cnt))) {
+            if ((err_info = sr_val_ly2sr(set->dnodes[i], 1, *values + *value_cnt))) {
                 goto cleanup;
             }
             ++(*value_cnt);
         }
     }
 
-    /* success */
 cleanup:
     ly_set_free(set, NULL);
     if (err_info) {
