@@ -4,29 +4,28 @@
  * @brief Sysrepo xpath util functions.
  *
  * @copyright
- * Copyright 2019 CESNET, z.s.p.o.
+ * Copyright (c) 2018 - 2021 Deutsche Telekom AG.
+ * Copyright (c) 2018 - 2021 CESNET, z.s.p.o.
  *
- * Licensed under the Apache License, Version 2.0 (the "License");
- * you may not use this file except in compliance with the License.
+ * This source code is licensed under BSD 3-Clause License (the "License").
+ * You may not use this file except in compliance with the License.
  * You may obtain a copy of the License at
  *
- *    http://www.apache.org/licenses/LICENSE-2.0
- *
- * Unless required by applicable law or agreed to in writing, software
- * distributed under the License is distributed on an "AS IS" BASIS,
- * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
- * See the License for the specific language governing permissions and
- * limitations under the License.
+ *     https://opensource.org/licenses/BSD-3-Clause
  */
 
+#define _GNU_SOURCE
+
+#include "xpath.h"
+
 #include <stdlib.h>
 #include <string.h>
 
 #include "common.h"
-#include "xpath.h"
+#include "config.h"
 
 static char *
-sr_get_next_node_internal(char *xpath, sr_xpath_ctx_t *state, bool skip_namespace)
+sr_get_next_node_internal(char *xpath, sr_xpath_ctx_t *state, int skip_namespace)
 {
     char *index = NULL, *quot = NULL;
 
@@ -93,13 +92,13 @@ sr_get_next_node_internal(char *xpath, sr_xpath_ctx_t *state, bool skip_namespac
 API char *
 sr_xpath_next_node(char *xpath, sr_xpath_ctx_t *state)
 {
-    return sr_get_next_node_internal(xpath, state, true);
+    return sr_get_next_node_internal(xpath, state, 1);
 }
 
 API char *
 sr_xpath_next_node_with_ns(char *xpath, sr_xpath_ctx_t *state)
 {
-    return sr_get_next_node_internal(xpath, state, false);
+    return sr_get_next_node_internal(xpath, state, 0);
 }
 
 API char *
@@ -569,7 +568,7 @@ sr_xpath_node_name(const char *xpath)
     return (char *)res;
 }
 
-API bool
+API int
 sr_xpath_node_name_eq(const char *xpath, const char *node_name)
 {
     char *xp_node_name = NULL;
@@ -577,7 +576,7 @@ sr_xpath_node_name_eq(const char *xpath, const char *node_name)
     xp_node_name = sr_xpath_node_name(xpath);
 
     if ((NULL == xp_node_name) || (NULL == node_name)) {
-        return false;
+        return 0;
     } else {
         return 0 == strcmp(xp_node_name, node_name);
     }
